﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;
using System.Threading;

// @Name: ao_waveform_pc_infinite.cs
// @Author: Smacq
// @Copyright: Smacq
// @Date: 2025/02/08
// @Brief:
// This code focuses on the Ao functions of the USB - 5000 device, performing waveform output.
// It outputs a -1V ~ 1V sine wave and a -5V ~ +5V square wave on Ao channel 0.
// The device's AO analog voltage range is -10V ~ +10V.
// The Ao channels vary depending on the module, either Ao_4 or NONE.
// You can refer to the user manual for more information about functions or device details.
//
// Execution result when only connecting USB for USB - 5100(Ao_4):
// 0
//...
// 0
// Ao0 outputting -1V and 1V sine wave, Please Enter to next wave.
// 0
// 0
// 0
// 0
// Ao0 outputting -1V and 1V square wave, Please Enter to stop output.
// 0 AOChannel_0 is 0V
// 0
// 0
// 0

class Program
{
    static void Main(string[] args)
    {
        // init -1 ~ 1V sine wave to Ao_data_array
        float[] Ao_data_array_sine = new float[1000];
        // init -1V and 1V square wave 200 data point to Ao_data_array
        float[] Ao_data_array_square = new float[200];

        // Device index starts from zero
        int DevIndex = 0;

        // Ao channel, Ao channel 0 to Ao channel 3
        byte AOChannel_0 = 0;
        byte AOChannel_1 = 1;
        byte AOChannel_2 = 2;
        byte AOChannel_3 = 3;

        // voltage value, range is -10 to +10
        float voltage_0 = 0.0f;

        // Ao channel sample mode:
        // 0 = continuous non - loop output to computer buffer
        // 1 = FIFO output loop determined by SetUSB5AoCycle()
        byte AoSampleMode = 1;

        // Ao Sampling period, unit ns, steps 10ns:
        // USB - 3000 is synchronous Ao, max rate is 100k = 10000.
        // sample rate and period:
        // 10k = 100000
        // 1k = 1000000
        // 100 = 10000000
        // 10 = 100000000
        // 1 = 1000000000
        uint AoSampleRate = 10000000;

        // Ao acquisition trig source:
        // 0 = SetUSB5AoSoftTrig(), 1 = Ext_Trig_In, 2~5 = Din_0~3,
        // 6 = with Ai, 7 = with Di, 7 = with Do, 9~12 = with AO~3, 255 = GlobalSoftTrig
        // Supported parameters: 0/1/2/3/4/5/6/7/8/9/10/11/12/255
        byte AoTrigSource = 0;

        // Ao acquisition clock source:
        // 0 = SetUSB5AoSampleRate(), 1 = Conv_In, 2 = Ai, 3 = Di, 5~8 = AO~3
        // Supported parameters: 0/1/2/3/5/6/7/8
        byte AoConvSource = 0;

        // Set waveform output cycle
        // 0 = infinite loop output
        // 1 = 1 time, 2 = 2 times and so on.
        // Supported parameters: 0 ~ 4294967295
        byte AoCycle = 0;

        // Download to Ao FIFO data number for sine wave
        uint Num_sine = 1000;
        // Download to Ao FIFO data number for square wave
        uint Num_square = 200;

        // Function to initialize sine wave data array
        void initSineWave(float[] array)
        {
            for (int i = 0; i < 1000; i++)
            {
                double x = -2 * Math.PI + (4 * Math.PI * i) / 999;
                array[i] = (float)Math.Sin(x);
            }
        }

        // Function to initialize square wave data array
        void initSquareWave(float[] array)
        {
            for (int i = 0; i < 100; i++)
            {
                array[i] = 1.0f;
            }
            for (int i = 100; i < 200; i++)
            {
                array[i] = -1.0f;
            }
        }


        int error_code;

        initSineWave(Ao_data_array_sine);

        // Open specific device and print return value.
        error_code = USB5000DLL.USB5OpenDevice(DevIndex);
        Console.WriteLine(error_code);

        // Disable Ao software trig, clear flag
        error_code = USB5000DLL.SetUSB5ClrAoTrigger(DevIndex, AOChannel_0);
        Console.WriteLine(error_code);

        // Clear FIFO
        error_code = USB5000DLL.SetUSB5ClrAoFifo(DevIndex, AOChannel_0);
        Console.WriteLine(error_code);

        // Set Device Ao sample mode
        error_code = USB5000DLL.SetUSB5AoSampleMode(DevIndex, AOChannel_0, AoSampleMode);
        Console.WriteLine(error_code);

        // Set Device sample rate
        error_code = USB5000DLL.SetUSB5AoSampleRate(DevIndex, AOChannel_0, AoSampleRate);
        Console.WriteLine(error_code);

        // Set Device trig source
        error_code = USB5000DLL.SetUSB5AoTrigSource(DevIndex, AOChannel_0, AoTrigSource);
        Console.WriteLine(error_code);

        // Set Device clock source
        error_code = USB5000DLL.SetUSB5AoConvSource(DevIndex, AOChannel_0, AoConvSource);
        Console.WriteLine(error_code);

        // Set digital output cycle 2 times
        error_code = USB5000DLL.SetUSB5AoCycle(DevIndex, AOChannel_0, AoCycle);
        Console.WriteLine(error_code);

        // Download array data to Ao FIFO for sine wave
        error_code = USB5000DLL.SetUSB5AoDataFifo(DevIndex, AOChannel_0, Ao_data_array_sine, Num_sine);
        Console.WriteLine(error_code);

        // Enable Ao software trig start Ao output wave
        error_code = USB5000DLL.SetUSB5AoSoftTrig(DevIndex, AOChannel_0);
        Console.WriteLine(error_code);

        Console.WriteLine("Ao0 outputting -1V and 1V sine wave, Please Enter to next wave.");
        Console.ReadLine();

        // Disable Ao software trig, clear flag
        error_code = USB5000DLL.SetUSB5ClrAoTrigger(DevIndex, AOChannel_0);
        Console.WriteLine(error_code);

        // Clear FIFO
        error_code = USB5000DLL.SetUSB5ClrAoFifo(DevIndex, AOChannel_0);
        Console.WriteLine(error_code);

        initSquareWave(Ao_data_array_square);

        // Download array data to Ao FIFO for square wave
        error_code = USB5000DLL.SetUSB5AoDataFifo(DevIndex, AOChannel_0, Ao_data_array_square, Num_square);
        Console.WriteLine(error_code);

        // Enable Ao software trig start Ao output wave
        error_code = USB5000DLL.SetUSB5AoSoftTrig(DevIndex, AOChannel_0);
        Console.WriteLine(error_code);

        Console.WriteLine("Ao0 outputting -1V and 1V square wave, Please Enter to stop output.");
        Console.ReadLine();

        error_code = USB5000DLL.SetUSB5AoImmediately(DevIndex, AOChannel_0, voltage_0);
        Console.WriteLine(error_code + " AOChannel_0 is 0V");

        // Disable Ao software trig, clear flag
        error_code = USB5000DLL.SetUSB5ClrAoTrigger(DevIndex, AOChannel_0);
        Console.WriteLine(error_code);

        // Clear FIFO
        error_code = USB5000DLL.SetUSB5ClrAoFifo(DevIndex, AOChannel_0);
        Console.WriteLine(error_code);

        // Close Device and stop Ao wave output
        error_code = USB5000DLL.USB5CloseDevice(DevIndex);
        Console.WriteLine(error_code);
    }

    class USB5000DLL
    {
        [DllImport("USB5000.dll")]
        public static extern int FindUSB5DAQ();
        [DllImport("USB5000.dll")]
        public static extern int USB5OpenDevice(int DevIndex);
        [DllImport("USB5000.dll")]
        public static extern int USB5CloseDevice(int DevIndex);
        [DllImport("USB5000.dll")]
        public static extern int USB5ReadDevcieSN(int DevIndex);
        [DllImport("USB5000.dll")]
        public static extern int USB5ReadDevcieModel(int DevIndex);

        //--------------------------------------------------------------------------
        // Ananlog Input Configuration

        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AiSampleRate(int DevIndex, uint SamplePeriod);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AiSampleMode(int DevIndex, byte AiSampleMode);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AiConnectType(int DevIndex, byte AiConnectType);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AiRange(int DevIndex, byte Chan, float AiRange);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AiChanSel(int DevIndex, byte Chan, byte Sel);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AiTrigSource(int DevIndex, byte AiTrigSource);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AiConvSource(int DevIndex, byte AiConvSource);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AiPreTrigPoints(int DevIndex, uint AiPreTrigPoints);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AiOneShotPoints(int DevIndex, uint AiOneShotPoints);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5ClrAiFifo(int DevIndex);

        //--------------------------------------------------------------------------
        // Digital I/O Configuration

        [DllImport("USB5000.dll")]
        public static extern int SetUSB5DiSampleRate(int DevIndex, uint SamplePeriod);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5DiSampleMode(int DevIndex, byte DiSampleMode);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5DiTrigSource(int DevIndex, byte DiTrigSource);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5DiConvSource(int DevIndex, byte DiConvSource);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5DiPreTrigPoints(int DevIndex, uint DiPreTrigPoints);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5DiOneShotPoints(int DevIndex, uint DiOneShotPoints);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5ClrDiFifo(int DevIndex);

        [DllImport("USB5000.dll")]
        public static extern int SetUSB5DoSampleRate(int DevIndex, uint SamplePeriod);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5DoSampleMode(int DevIndex, byte DoSampleMode);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5DoTrigSource(int DevIndex, byte DoTrigSource);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5DoConvSource(int DevIndex, byte DoConvSource);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5DoCycle(int DevIndex, uint DoCycle);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5DoDataFifo(int DevIndex, uint[] Value, uint Len);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5ClrDoFifo(int DevIndex);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5DoWaveCtrl(int DevIndex, uint Chan);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5DoImmediately(int DevIndex, uint Chan, uint Value);

        //--------------------------------------------------------------------------
        // Ananlog Output Configuration

        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AoSampleRate(int DevIndex, byte Chan, uint SamplePeriod);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AoSampleMode(int DevIndex, byte Chan, byte AoSampleMode);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AoTrigSource(int DevIndex, byte Chan, byte AoTrigSource);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AoConvSource(int DevIndex, byte Chan, byte AoConvSource);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AoCycle(int DevIndex, byte Chan, byte AoCycle);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AoDataFifo(int DevIndex, byte Chan, float[] Voltage, uint Len);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5ClrAoFifo(int DevIndex, byte Chan);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AoSync(int DevIndex, byte Chans);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AoImmediately(int DevIndex, byte Chan, float Voltage);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AoImmediately4Chans(int DevIndex, float Volt0, float Volt1, float Volt2, float Volt3);

        //--------------------------------------------------------------------------
        // Trig Control

        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AiSoftTrig(int DevIndex);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5DiSoftTrig(int DevIndex);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5DoSoftTrig(int DevIndex);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AoSoftTrig(int DevIndex, byte Chan);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5GlobalSoftTrig(int DevIndex);

        [DllImport("USB5000.dll")]
        public static extern int SetUSB5ClrTrigger(int DevIndex);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5ClrAiTrigger(int DevIndex);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5ClrDiTrigger(int DevIndex);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5ClrDoTrigger(int DevIndex);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5ClrAoTrigger(int DevIndex, byte Chan);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5ClrGlobalSoftTrig(int DevIndex);

        //--------------------------------------------------------------------------
        // Sync Configuration

        [DllImport("USB5000.dll")]
        public static extern int SetUSB5ExtTrigOutSource(int DevIndex, byte Source);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5ExtConvOutSource(int DevIndex, byte Source);

        //--------------------------------------------------------------------------
        // Get Data Acquired

        [DllImport("USB5000.dll")]
        public static extern int USB5GetAi(int DevIndex, uint Points, float[] Ai, int TimeOut);
        [DllImport("USB5000.dll")]
        public static extern int USB5GetDi(int DevIndex, uint Points, byte[] Di, int TimeOut);
    }
}