﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;
using System.Threading;


// @Name: di_continuous.cs
// @Author: Smacq
// @Copyright: Smacq
// @Date: 2025/02/08
// @Brief:
// This code focuses on the Di functions of the USB - 5000 device, performing continuous digital input.
// It acquires 100 data points from the Di channel 10 times.
// The DI default suspended is 3.3V HIGH, and 0 - 1.2V is LOW.
// DI channel is different for modul, DI_2 or DI_4 or DI_8.
// You can refer to the user manual for more information about functions or device details.
//
// Execution result when only connecting USB for USB - 5000(DI_2):
// 0
// 0
// 0
// 0
// 0
// 0
// 0
// remain_buffer: 2
// Di_channel 100 data points
//  [3 3 3 3 3 3 3 3 3 3 3 3 3 3 3 3 3 3 3 3 3 3 3 3
//...
//  3 3 3 3]
// 0
// 0
// 0

class Program
{
    static void Main(string[] args)
    {
        // Device index starts from zero
        int DevIndex = 0;

        // Init Di_data_array
        // Usually the array size need >= acquisition data points size.
        // The data points saved to Di_data_array arrange from opened Di channel
        // Data type is uint8, it is not modifiable.
        // Di_channel_4:0~0xF(1111) Di3~DI0, Di_channel_8:0~0xFF(1111 1111) Di7~DI0.
        byte[] Di_data_array = new byte[100];

        // Di channel sample mode:
        // 0 = Continuous acquisition or 1 = Oneshot, param support 0/1
        byte DiSampleMode = 0;

        // Acquisition amount of data:
        // Each opened Di channel will get this amount of data.
        // Suggest value within sampling rate 10%~20%.
        uint Points = 100;

        // Di Sampling period, unit ns, steps 10ns:
        // USB - 3000 is synchronous DI, channel max rate is 10M = 100.
        // sample rate and period:
        // 1M = 1000
        // 100k = 10000
        // 10k = 100000
        // 1k = 1000000
        // 100 = 10000000
        // 10 = 100000000
        // 1 = 1000000000
        uint DiSamplePeriod = 1000000;

        // Di acquisition trig source:
        // 0 = SetUSB5DiSoftTrig(), 1 = Ext_Trig_In, 2~5 = Din_0~3,
        // 6 = with Di, 8 = with DO, 9~12 = with AO~3, 255 = GlobalSoftTrig
        // Supported parameters: 0/1/2/3/4/5/6/8/9/11/12/255
        byte DiTrigSource = 0;

        // Di acquisition clock source:
        // 0 = SetUSB5DiSampleRate(), 1 = Conv_In, 2 = Di, 4 = DO, 5~8 = AO~3
        // Supported parameters: 0/1/2/4/5/6/7/8
        byte DiConvSource = 0;

        // Timeout unit ms:
        // Suggest Timeout>(Num*Channel_count/Samplerate)*1000
        // Avoid USB5GetDi() func timeout, if timeout return -7.
        int Timeout = 200;

        int error_code;

        // Open specific device and print return value.
        error_code = USB5000DLL.USB5OpenDevice(DevIndex);
        Console.WriteLine(error_code);

        // Set Device Di sample mode
        error_code = USB5000DLL.SetUSB5DiSampleMode(DevIndex, DiSampleMode);
        Console.WriteLine(error_code);

        // Set Device sample rate
        error_code = USB5000DLL.SetUSB5DiSampleRate(DevIndex, DiSamplePeriod);
        Console.WriteLine(error_code);

        // Set Device clock source
        error_code = USB5000DLL.SetUSB5DiConvSource(DevIndex, DiConvSource);
        Console.WriteLine(error_code);

        // Clear Di FIFO
        error_code = USB5000DLL.SetUSB5ClrDiFifo(DevIndex);
        Console.WriteLine(error_code);

        // Set Device trig source
        error_code = USB5000DLL.SetUSB5DiTrigSource(DevIndex, DiTrigSource);
        Console.WriteLine(error_code);

        // Enable Di software trig acquisition
        error_code = USB5000DLL.SetUSB5DiSoftTrig(DevIndex);
        Console.WriteLine(error_code);

        // Continuous acquisition execution 10 times GetDiChans()
        for (int i = 0; i < 10; ++i)
        {
            uint remain_buffer = (uint)USB5000DLL.USB5GetDi(DevIndex, Points, Di_data_array, Timeout);
            /*
            remain_buffer range 0~2000000
            USB5GetDi() return the remaining Device FIFO buffer value.
            FIFO buffer 100 data points save to Di_data_array.
            Channel_0 100 data points is Di_data_array[0:100]
            If remain_buffer increase you should run USB5GetDi() more quick, let remain_buffer steady.
            */
            Console.WriteLine("\nremain_buffer: " + remain_buffer);
            Console.WriteLine("Di_channel 100 data points\n");
            for (int j = 0; j < 100; ++j)
            {
                Console.Write(Di_data_array[j] + " ");
            }
        }
        Console.WriteLine();

        // Disable Di software trig acquisition
        error_code = USB5000DLL.SetUSB5ClrDiTrigger(DevIndex);
        Console.WriteLine(error_code);

        // Clear Di FIFO
        error_code = USB5000DLL.SetUSB5ClrDiFifo(DevIndex);
        Console.WriteLine(error_code);

        // Close Device
        error_code = USB5000DLL.USB5CloseDevice(DevIndex);
        Console.WriteLine(error_code);
    }

    class USB5000DLL
    {
        [DllImport("USB5000.dll")]
        public static extern int FindUSB5DAQ();
        [DllImport("USB5000.dll")]
        public static extern int USB5OpenDevice(int DevIndex);
        [DllImport("USB5000.dll")]
        public static extern int USB5CloseDevice(int DevIndex);
        [DllImport("USB5000.dll")]
        public static extern int USB5ReadDevcieSN(int DevIndex);
        [DllImport("USB5000.dll")]
        public static extern int USB5ReadDevcieModel(int DevIndex);

        //--------------------------------------------------------------------------
        // Ananlog Input Configuration

        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AiSampleRate(int DevIndex, uint SamplePeriod);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AiSampleMode(int DevIndex, byte AiSampleMode);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AiConnectType(int DevIndex, byte AiConnectType);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AiRange(int DevIndex, byte Chan, float AiRange);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AiChanSel(int DevIndex, byte Chan, byte Sel);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AiTrigSource(int DevIndex, byte AiTrigSource);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AiConvSource(int DevIndex, byte AiConvSource);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AiPreTrigPoints(int DevIndex, uint AiPreTrigPoints);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AiOneShotPoints(int DevIndex, uint AiOneShotPoints);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5ClrAiFifo(int DevIndex);

        //--------------------------------------------------------------------------
        // Digital I/O Configuration

        [DllImport("USB5000.dll")]
        public static extern int SetUSB5DiSampleRate(int DevIndex, uint SamplePeriod);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5DiSampleMode(int DevIndex, byte DiSampleMode);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5DiTrigSource(int DevIndex, byte DiTrigSource);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5DiConvSource(int DevIndex, byte DiConvSource);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5DiPreTrigPoints(int DevIndex, uint DiPreTrigPoints);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5DiOneShotPoints(int DevIndex, uint DiOneShotPoints);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5ClrDiFifo(int DevIndex);

        [DllImport("USB5000.dll")]
        public static extern int SetUSB5DoSampleRate(int DevIndex, uint SamplePeriod);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5DoSampleMode(int DevIndex, byte DoSampleMode);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5DoTrigSource(int DevIndex, byte DoTrigSource);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5DoConvSource(int DevIndex, byte DoConvSource);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5DoCycle(int DevIndex, uint DoCycle);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5DoDataFifo(int DevIndex, uint[] Value, uint Len);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5ClrDoFifo(int DevIndex);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5DoWaveCtrl(int DevIndex, uint Chan);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5DoImmediately(int DevIndex, uint Chan, uint Value);

        //--------------------------------------------------------------------------
        // Ananlog Output Configuration

        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AoSampleRate(int DevIndex, byte Chan, uint SamplePeriod);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AoSampleMode(int DevIndex, byte Chan, byte AoSampleMode);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AoTrigSource(int DevIndex, byte Chan, byte AoTrigSource);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AoConvSource(int DevIndex, byte Chan, byte AoConvSource);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AoCycle(int DevIndex, byte Chan, byte AoCycle);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AoDataFifo(int DevIndex, byte Chan, float[] Voltage, uint Len);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5ClrAoFifo(int DevIndex, byte Chan);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AoSync(int DevIndex, byte Chans);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AoImmediately(int DevIndex, byte Chan, float Voltage);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AoImmediately4Chans(int DevIndex, float Volt0, float Volt1, float Volt2, float Volt3);

        //--------------------------------------------------------------------------
        // Trig Control

        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AiSoftTrig(int DevIndex);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5DiSoftTrig(int DevIndex);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5DoSoftTrig(int DevIndex);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AoSoftTrig(int DevIndex, byte Chan);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5GlobalSoftTrig(int DevIndex);

        [DllImport("USB5000.dll")]
        public static extern int SetUSB5ClrTrigger(int DevIndex);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5ClrAiTrigger(int DevIndex);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5ClrDiTrigger(int DevIndex);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5ClrDoTrigger(int DevIndex);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5ClrAoTrigger(int DevIndex, byte Chan);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5ClrGlobalSoftTrig(int DevIndex);

        //--------------------------------------------------------------------------
        // Sync Configuration

        [DllImport("USB5000.dll")]
        public static extern int SetUSB5ExtTrigOutSource(int DevIndex, byte Source);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5ExtConvOutSource(int DevIndex, byte Source);

        //--------------------------------------------------------------------------
        // Get Data Acquired

        [DllImport("USB5000.dll")]
        public static extern int USB5GetAi(int DevIndex, uint Points, float[] Ai, int TimeOut);
        [DllImport("USB5000.dll")]
        public static extern int USB5GetDi(int DevIndex, uint Points, byte[] Di, int TimeOut);
    }
}
