﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;
using System.Threading;

// @Name: do_waveform_pc_infinite.cs
// @Author: Smacq
// @Copyright: Smacq
// @Date: 2025/02/08
// @Brief:
// This code focuses on the Do functions of the USB - 5000 device, enabling two types of digital waveform output.
// It sets the Do output to HIGH/LOW, where 0V is LOW (default) and 3.3V is HIGH.
// The DO channels vary depending on the module, either Do_2 or Do_4 or Do_8.
// You can refer to the user manual for more information about functions or device details.
//
// Execution result when only connecting USB for USB - 5000:
// 0
//...
// 0
// Do0~Do7 Flowing HIGH Interval of 1s, Please Enter to new wave.
// 0
// 0
// 0
// 0
// Do0~Do7 HiHG/LOW Interval of 1s, Please Enter to stop Do output.
// 0
// 0
// 0


class Program
{
    static void Main(string[] args)
    {
        // Device index starts from zero
        int DevIndex = 0;

        // Initialize the Do data array
        // The data type is uint32 and it is not modifiable.
        // Do data point 0 represents LOW, and 1 represents HIGH.
        // For example:
        // Do_channel_4: 0xF (1111) means Do3~Do0 are HIGH
        // Do_channel_8: 0xFF (1111 1111) means Do7~Do0 are HIGH
        // Do_channel_4: 0x3 (0011) means Do3~Do2 are LOW, and Do1~Do0 are HIGH
        // Do_channel_8: 0x88 (1000 1000) means Do7 and Do3 are HIGH, and other Dos are LOW
        int firstDataPoints = 8;
        uint[] Do_data_array_first = { 1, 2, 4, 8, 16, 32, 64, 128 };

        int secondDataPoints = 2;
        uint[] Do_data_array_second = { 0x0, 0xFF };

        // Select digital output wave channel
        // Do_channel_4: 0 (0000) ~ 0xF (1111) corresponds to Do3~Do0
        // Do_channel_8: 0 (0000 0000) ~ 0xFF (1111 1111) corresponds to Do7~Do0
        // Examples of selecting Do channels:
        // Do_0: 0x1 (0001), Do_0 and Do_1: 0x3 (0011), Do_3 and Do_7: 0x88 (1000 1000)
        uint DoWaveChannel = 0xFF;

        // Do channel sample mode
        // 0 = Continuous non - loop output to the computer buffer
        // 1 = FIFO output loop determined by SetUSB5DoCycle()
        byte DoSampleMode = 1;

        // Do sampling period, unit ns, steps 10ns
        // USB - 3000 is synchronous Do, and the maximum rate is 10M = 100.
        // Sampling rate and period:
        // 1M = 1000
        // 100k = 10000
        // 10k = 100000
        // 1k = 1000000
        // 100 = 10000000
        // 10 = 100000000
        // 1 = 1000000000
        uint DoSampleRate = 1000000000;

        // Do acquisition trig source
        // 0 = SetUSB5AiSoftTrig(), 1 = Ext_Trig_In, 2~5 = Din_0~3,
        // 6 = with AI, 7 = with DI, 9~12 = with AO~3, 255 = GlobalSoftTrig
        // Supported parameters: 0/1/2/3/4/5/6/7/9/11/12/255
        byte DoTrigSource = 0;

        // Do acquisition clock source
        // 0 = SetUSB5DoSampleRate(), 1 = Conv_In, 2 = Ai, 3 = Di, 5~8 = AO~3
        // Supported parameters: 0/1/2/3/5/6/7/8
        byte DoConvSource = 0;

        // Set waveform output cycle
        // 0 = Infinite loop output
        // 1 = 1 time, 2 = 2 times, and so on.
        // Supported parameters: 0 ~ 4294967295
        uint DoCycle = 0;

        int error_code;

        // Open specific device and print return value
        error_code = USB5000DLL.USB5OpenDevice(DevIndex);
        Console.WriteLine(error_code);

        // Disable Do software trig, clear flag
        error_code = USB5000DLL.SetUSB5ClrDoTrigger(DevIndex);
        Console.WriteLine(error_code);

        // Clear FIFO
        error_code = USB5000DLL.SetUSB5ClrDoFifo(DevIndex);
        Console.WriteLine(error_code);

        // Set Device Do sample mode
        error_code = USB5000DLL.SetUSB5DoSampleMode(DevIndex, DoSampleMode);
        Console.WriteLine(error_code);

        // Set Device sample rate
        error_code = USB5000DLL.SetUSB5DoSampleRate(DevIndex, DoSampleRate);
        Console.WriteLine(error_code);

        // Set digital output wave channel
        error_code = USB5000DLL.SetUSB5DoWaveCtrl(DevIndex, DoWaveChannel);
        Console.WriteLine(error_code);

        // Set digital output cycle infinite loop
        error_code = USB5000DLL.SetUSB5DoCycle(DevIndex, DoCycle);
        Console.WriteLine(error_code);

        // Set Device trig source
        error_code = USB5000DLL.SetUSB5DoTrigSource(DevIndex, DoTrigSource);
        Console.WriteLine(error_code);

        // Set Device clock source
        error_code = USB5000DLL.SetUSB5DoConvSource(DevIndex, DoConvSource);
        Console.WriteLine(error_code);

        // Download array data to Do FIFO
        error_code = USB5000DLL.SetUSB5DoDataFifo(DevIndex, Do_data_array_first, (uint)firstDataPoints);
        Console.WriteLine(error_code);

        // Enable Do software trig start DO output wave
        error_code = USB5000DLL.SetUSB5DoSoftTrig(DevIndex);
        Console.WriteLine(error_code);

        // Wait for user input
        Console.WriteLine("Do0~Do7 Flowing HIGH Interval of 1s, Please Enter to new wave.");
        Console.ReadKey();

        // Disable Do software trig, clear flag
        error_code = USB5000DLL.SetUSB5ClrDoTrigger(DevIndex);
        Console.WriteLine(error_code);

        // Clear FIFO
        error_code = USB5000DLL.SetUSB5ClrDoFifo(DevIndex);
        Console.WriteLine(error_code);

        // Update new wave
        // Download array data to Do FIFO
        error_code = USB5000DLL.SetUSB5DoDataFifo(DevIndex, Do_data_array_second, (uint)secondDataPoints);
        Console.WriteLine(error_code);

        // Enable Do software trig start DO output wave
        error_code = USB5000DLL.SetUSB5DoSoftTrig(DevIndex);
        Console.WriteLine(error_code);

        // Wait for user input
        Console.WriteLine("Do0~Do7 HiHG/LOW Interval of 1s, Please Enter to stop Do output.");
        Console.ReadKey();

        // Disable Do software trig, clear flag
        error_code = USB5000DLL.SetUSB5ClrDoTrigger(DevIndex);
        Console.WriteLine(error_code);

        // Clear FIFO
        error_code = USB5000DLL.SetUSB5ClrDoFifo(DevIndex);
        Console.WriteLine(error_code);

        // Close Device and stop Do wave output
        error_code = USB5000DLL.USB5CloseDevice(DevIndex);
        Console.WriteLine(error_code);
    }
    class USB5000DLL
    {
        [DllImport("USB5000.dll")]
        public static extern int FindUSB5DAQ();
        [DllImport("USB5000.dll")]
        public static extern int USB5OpenDevice(int DevIndex);
        [DllImport("USB5000.dll")]
        public static extern int USB5CloseDevice(int DevIndex);
        [DllImport("USB5000.dll")]
        public static extern int USB5ReadDevcieSN(int DevIndex);
        [DllImport("USB5000.dll")]
        public static extern int USB5ReadDevcieModel(int DevIndex);
        //--------------------------------------------------------------------------
        // Ananlog Input Configuration
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AiSampleRate(int DevIndex, uint SamplePeriod);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AiSampleMode(int DevIndex, byte AiSampleMode);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AiConnectType(int DevIndex, byte AiConnectType);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AiRange(int DevIndex, byte Chan, float AiRange);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AiChanSel(int DevIndex, byte Chan, byte Sel);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AiTrigSource(int DevIndex, byte AiTrigSource);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AiConvSource(int DevIndex, byte AiConvSource);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AiPreTrigPoints(int DevIndex, uint AiPreTrigPoints);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AiOneShotPoints(int DevIndex, uint AiOneShotPoints);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5ClrAiFifo(int DevIndex);
        //--------------------------------------------------------------------------
        // Digital I/O Configuration
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5DiSampleRate(int DevIndex, uint SamplePeriod);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5DiSampleMode(int DevIndex, byte DiSampleMode);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5DiTrigSource(int DevIndex, byte DiTrigSource);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5DiConvSource(int DevIndex, byte DiConvSource);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5DiPreTrigPoints(int DevIndex, uint DiPreTrigPoints);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5DiOneShotPoints(int DevIndex, uint DiOneShotPoints);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5ClrDiFifo(int DevIndex);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5DoSampleRate(int DevIndex, uint SamplePeriod);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5DoSampleMode(int DevIndex, byte DoSampleMode);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5DoTrigSource(int DevIndex, byte DoTrigSource);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5DoConvSource(int DevIndex, byte DoConvSource);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5DoCycle(int DevIndex, uint DoCycle);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5DoDataFifo(int DevIndex, uint[] Value, uint Len);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5ClrDoFifo(int DevIndex);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5DoWaveCtrl(int DevIndex, uint Chan);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5DoImmediately(int DevIndex, uint Chan, uint Value);
        //--------------------------------------------------------------------------
        // Ananlog Output Configuration
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AoSampleRate(int DevIndex, byte Chan, uint SamplePeriod);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AoSampleMode(int DevIndex, byte Chan, byte AoSampleMode);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AoTrigSource(int DevIndex, byte Chan, byte AoTrigSource);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AoConvSource(int DevIndex, byte Chan, byte AoConvSource);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AoCycle(int DevIndex, byte Chan, byte AoCycle);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AoDataFifo(int DevIndex, byte Chan, float[] Voltage, uint Len);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5ClrAoFifo(int DevIndex, byte Chan);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AoSync(int DevIndex, byte Chans);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AoImmediately(int DevIndex, byte Chan, float Voltage);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AoImmediately4Chans(int DevIndex, float Volt0, float Volt1, float Volt2, float Volt3);
        //--------------------------------------------------------------------------
        // Trig Control
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AiSoftTrig(int DevIndex);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5DiSoftTrig(int DevIndex);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5DoSoftTrig(int DevIndex);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AoSoftTrig(int DevIndex, byte Chan);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5GlobalSoftTrig(int DevIndex);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5ClrTrigger(int DevIndex);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5ClrAiTrigger(int DevIndex);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5ClrDiTrigger(int DevIndex);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5ClrDoTrigger(int DevIndex);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5ClrAoTrigger(int DevIndex, byte Chan);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5ClrGlobalSoftTrig(int DevIndex);
        //--------------------------------------------------------------------------
        // Sync Configuration
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5ExtTrigOutSource(int DevIndex, byte Source);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5ExtConvOutSource(int DevIndex, byte Source);
        //--------------------------------------------------------------------------
        // Get Data Acquired
        [DllImport("USB5000.dll")]
        public static extern int USB5GetAi(int DevIndex, uint Points, float[] Ai, int TimeOut);
        [DllImport("USB5000.dll")]
        public static extern int USB5GetDi(int DevIndex, uint Points, byte[] Di, int TimeOut);
    }
}